/**
 * @file imganalysis.c
 * @brief Provides functions to analyze images
 * @author Florian Schmitt (0schmitt@informatik.uni-hamburg.de)
 * @date Aug-2012
**/
#include "camcapture.h"
#include <stdlib.h>




/**
 * Detects edges on the x-Plane
 * @param image  Image data pointer to image_data struct
 * @param thres  Intensity (Y) threshold value
 * @param max    Maximum ammount of edges to detect (typ. the length of image_point array)
 * @param raster Raster which is gone through
 * @param list   List of image_point structs to fill found points in
 * @return number of edge-points found
**/
int camcapture_edge_detection_x(image_data_t *image, int thres, int raster, int max, image_point_list_t *list)
{
	int cnt = 0;
	int scan_x, scan_y, Y0, Y1, value, tmp;
	void *data = image->img;
	image_point_t *point;
	/* scan image for possible circle objects */
	for(scan_y=0; (scan_y < image->height); scan_y += raster)
	{
		/* scan line */
		for(scan_x=0; (scan_x < (image->width-4)); scan_x++)
		{
			/* break if maximum points reached */
			if(cnt >= max) return cnt;
			/* get first values median */
			value = *((int*)data+((scan_y*image->width)+scan_x)/2);
			Y0 = (int)(value & 0xFF);
			Y1 = (int)((value >> 16) & 0xFF);
			tmp = (Y0+Y1) / 2;
			/* get second values */
			value = *((int*)data+((scan_y*image->width)+scan_x+2)/2);
			Y0 = (int)(value & 0xFF);
			Y1 = (int)((value >> 16) & 0xFF);
			/* check */
			if( absolute(tmp - ((Y0+Y1) / 2)) > thres )
			{
				/* allocade and add point to list */
				point = malloc(sizeof(struct image_point));
				point->x = scan_x;
				point->y = scan_y;
				list = camcapture_add_point_list(list, point);
				cnt++;
			}
		}
	}
	return cnt;
}







/**
 * Detects edges on the x-Plane of a given color
 * Does not need to have a filter applied before
 * @param image  Image data pointer to image_data struct
 * @param color  Filter color to use
 * @param sens   Sensibility for the color filter
 * @param max    Maximum ammount of edges to detect (typ. the length of image_point array)
 * @param raster Raster which is gone through
 * @param list   List of image_point structs to fill found points in
 * @return number of edge-points found
**/
int camcapture_edge_detection_x_col(image_data_t *image, color_rgb_t *color, int sens, int raster, int max, image_point_list_t *list)
{
	int cnt = 0;
	int scan_x, scan_y, is1, is2, FCb, FCr;
	void *data = image->img;
	void *ptr;
	image_point_t *point;
	/* convert color */
	rgb_to_ycbcr(color, 0, &FCb, &FCr);
	/* scan image for possible circle objects */
	for(scan_y=0; (scan_y < image->height); scan_y += raster)
	{
		/* scan line */
		for(scan_x=0; (scan_x < (image->width-4)); scan_x++)
		{
			/* break if maximum points reached */
			if(cnt >= max) return cnt;
			/* get first values median */
			ptr = (void *) ((int*)data + ((scan_y*image->width)+scan_x)/2);
			is1 = camcapture_filter_color_single(ptr, FCb, FCr, sens);
			/* get second value */
			ptr = (void *) ((int*)data + ((scan_y*image->width)+scan_x+2)/2);
			is2 = camcapture_filter_color_single(ptr, FCb, FCr, sens);
			/* check */
			if( is1 ^ is2 )
			{
				
				/* allocate memory and add point to list */
				point = malloc(sizeof(struct image_point));
				point->x = scan_x;
				point->y = scan_y;
				list = camcapture_add_point_list(list, point);
				/* increase added points counter */
				cnt++;
			}
		}
	}
	return cnt;
}







/**
 * Detects edges on the x-Plane of a given luma delta
 * Does not need to have a filter applied before
 * @param image  Image data pointer to image_data struct
 * @param dy     luma difference to detect edges with
 * @param sens   Sensibility for the color filter
 * @param max    Maximum ammount of edges to detect (typ. the length of image_point array)
 * @param raster Raster which is gone through
 * @param list   List of image_point structs to fill found points in
 * @return number of edge-points found
**/
int camcapture_edge_detection_x_dlum(image_data_t *image, color_rgb_t *dy, int sens, int raster, int max, image_point_list_t *list)
{
	int cnt = 0;
	int scan_x, scan_y, is1, is2;
	void *data = image->img;
	void *ptr;
	image_point_t *point;
	/* scan image for possible circle objects */
	for(scan_y=0; (scan_y < image->height); scan_y += raster)
	{
		/* scan line */
		for(scan_x=0; (scan_x < (image->width-4)); scan_x = scan_x+2)
		{
			/* break if maximum points reached */
			if(cnt >= max) return cnt;
			/* get first values median */
			ptr  = (void *) ((int*)data + ((scan_y*image->width)+scan_x)/2);
			is1 = camcapture_filter_luma_single(ptr, sens);
			/* get second value */
			ptr  = (void *) ((int*)data + ((scan_y*image->width)+scan_x+2)/2);
			is2 = camcapture_filter_luma_single(ptr, sens);
			/* check */
			if( is1 ^ is2 )
			{
				/* allocate memory and add point to list */
				point = malloc(sizeof(struct image_point));
				point->x = scan_x;
				point->y = scan_y;
				list = camcapture_add_point_list(list, point);
				/* increase added points counter */
				cnt++;
			}
		}
	}
	return cnt;
}




/**
 * Rect threshold
 * Adds all luminance values in rects and adds point if it exceeds threshold
 * @param image  Image data pointer to image_data struct
 * @param thres  Intensity (Y) threshold value (0-255)
 * @param raster Raster for the rect (x=y)
 * @param list   List of image_point structs to fill found rect-centers in
 * @return number of edge-points found
**/
int camcapture_rect_threshold(image_data_t *image, int thres, int raster, image_point_list_t *list)
{
	int cnt = 0;
	int scan_x, scan_y, Y0, Y1, value, rx, ry;
	long tmp;
	void *data = image->img;
	image_point_t *point;
	int r_half = raster / 2;
	int r_quad = raster * raster;
	/* scan image for possible circle objects */
	for(scan_y=0; (scan_y < image->height-raster); scan_y += raster)
	{
		/* scan line */
		for(scan_x=0; (scan_x < (image->width-raster)); scan_x += raster)
		{
			/* Step through the rect */
			tmp = 0;
			for(rx=0; rx<raster; rx++)
				for(ry=0; ry<raster; ry++)
					{
						value = *((int*)data+((scan_y*image->width)+scan_x)/2);
						Y0 = (int)(value & 0xFF);
						Y1 = (int)((value >> 16) & 0xFF);
						tmp += Y0 + Y1;
					}
			/* check */
			if( (tmp/(r_quad)) > thres )
			{
				/* allocade and add point to list */
				point = malloc(sizeof(struct image_point));
				point->x = scan_x + r_half;
				point->y = scan_y + r_half;
				list = camcapture_add_point_list(list, point);
				cnt++;
			}
		}
	}
	return cnt;
}
