/**
 * @file imgfx.c
 * @brief Some tests to see what it works like
 * @author Florian Schmitt (0schmitt@informatik.uni-hamburg.de)
 * @date Jul-2012
**/
#include "camcapture.h"





/**
 * Checks a point for a specific color, DOES NOT MODIFY THE IMAGE
 * @param ptr    Image points data pointer
 * @param FCb    Cb value to filter
 * @param FCr    Cr value to filter
 * @param sens   Sensibility threshold
 * @return 1 if color matched, 0 if not
**/
int camcapture_filter_color_single(void *ptr, int FCb, int FCr, int sens)
{
	int value, Cb, Cr;
	value = *((int*)ptr);
	/* get values from image point */
	Cb = (int)((value >> 8) & 0xFF);
	Cr = (int)((value >> 24) & 0xFF);
	/* check filter */
	if( (absolute(FCr-Cr) < sens) && (absolute(FCb-Cb) < sens) ) return 1;
	return 0;
}



/**
 * Filter a color
 * @param image  Image data pointer to image_data struct
 * @param color  RGB-Color code to filter
 * @param sens   Sensibility threshold
**/
void camcapture_filter_color(image_data_t *image, color_rgb_t *color, int sens)
{
	int value, Y0, Cb, Y1, Cr, FCb, FCr;
	unsigned int i, max;
	void *data = image->img;
	/* Get the filter-colors FCb and FCr value */
	rgb_to_ycbcr(color, 0, &FCb, &FCr);
	/* check all pixels */
	max = (image->width * image->height/2);
	for(i=0; i<max; i++)
	{
		/* get vaulues */
		value = *((int*)data+i);
		Y0 = (int)(value & 0xFF);
		Cb = (int)((value >> 8) & 0xFF);
		Y1 = (int)((value >> 16) & 0xFF);
		Cr = (int)((value >> 24) & 0xFF);
		/* check filter */
		if( (absolute(FCr-Cr) < sens) && (absolute(FCb-Cb) < sens) ) { Y0 = 255; Y1 = 255; Cr=128; Cb=128; }
		else { Y0 = 0; Y1 = 0; Cr=128; Cb=128; }
		/* combine new value and write to buffer*/
		*((int*)data+i) = 0 | (Y0) | (Cb << 8) | (Y1 << 16) | (Cr << 24);
	}
}




/**
 * Returns a points luma
 * @param ptr    Image points data pointer
 * @return 1 if color matched, 0 if not
**/
int camcapture_get_luma_single(void *ptr)
{
	int value, Y0, Y1;
	value = *((int*)ptr);
	/* get values from image point */
	Y0 = (int)(value & 0xFF);
	Y1 = (int)((value >> 16) & 0xFF);
	/* calculate delta */
	return (Y0 + Y1)/2;
}



/**
 * 1 if luma exceeds sens
 * @param ptr    Image points data pointer
 * @param sens   Sensibility threshold
 * @return 1 if color matched, 0 if not
**/
int camcapture_filter_luma_single(void *ptr, int sens)
{
	int value, Y0, Y1;
	value = *((int*)ptr);
	/* get values from image point */
	Y0 = (int)(value & 0xFF);
	Y1 = (int)((value >> 16) & 0xFF);
	/* calculate delta */
	return absolute((Y0 + Y1)/2) >= sens;
}




/**
 * Filter by luminescense
 * @param image  Image data pointer to image_data struct
 * @param thres  Luminescense threshold
**/
void camcapture_filter_lumi(image_data_t *image, int thres)
{
	int value, Y0, Cb, Y1, Cr;
	unsigned int i, max;
	void *data = image->img;
	/* check all pixels */
	max = (image->width * image->height/2);
	for(i=0; i<max; i++)
	{
		/* get vaulues */
		value = *((int*)data+i);
		Y0 = (int)(value & 0xFF);
		Cb = (int)((value >> 8) & 0xFF);
		Y1 = (int)((value >> 16) & 0xFF);
		Cr = (int)((value >> 24) & 0xFF);
		/* check filter */
		if( (Y0 < thres) || (Y1 < thres) ) { Y0 = 255; Y1 = 255; Cr=128; Cb=128; }
		else { Y0 = 0; Y1 = 0; Cr=128; Cb=128; }
		/* combine new value and write to buffer*/
		*((int*)data+i) = 0 | (Y0) | (Cb << 8) | (Y1 << 16) | (Cr << 24);
	}
}




/**
 * Generates a new Image which shows the difference between two pictures
 * ALL IMAGES MUST BE OF THE SAME SIZE!
 * destination may be the same object than src1
 * Sensibility is the minimum value to be changes (e.g. if 20, all points < 20 will be ignored)
 * @param destination  Image data pointer to image_data struct - must be an instanciated image!
 * @param src1         First source image
 * @param src2         Second source image
 * @param sens         Sensibility threshold
**/
void camcapture_filter_difference(image_data_t *destination, image_data_t *src1, 
                                  image_data_t *src2, int sens)
{
	int value, Y0, Cb, Y1, Cr;
	unsigned int i, max;
	void *destination_i = destination->img;
	void *src1_i = src1->img;
	void *src2_i = src2->img;
	/* check all pixels */
	max = (src1->width * src1->height/2);
	for(i=0; i<max; i++)
	{
		/* get vaulues of the first picture */
		value = *((int*)src1_i+i);
		Y0 = (int)(value & 0xFF);
		Cb = (int)((value >> 8) & 0xFF);
		Y1 = (int)((value >> 16) & 0xFF);
		Cr = (int)((value >> 24) & 0xFF);
		/* get values of the second picture and get difference */
		value = *((int*)src2_i+i);
		Y0 = absolute( Y0 - (int)(value & 0xFF) );
		Cb = absolute( Cb - (int)((value >> 8) & 0xFF) );
		Y1 = absolute( Y1 - (int)((value >> 16) & 0xFF) );
		Cr = absolute( Cr - (int)((value >> 24) & 0xFF) );
		/* calculate new values */
		value = clamp(Y0 + Cb + Y1 + Cr);
		if( value < sens ) value = 0;
		/* combine new value and write to buffer*/
		*((int*)destination_i+i) = 0 | (value) | (128 << 8) | (value << 16) | (128 << 24);
	}
}
